/*
 * Pidgin Authorization Greylisting
 * Copyright (C) 2011 Konrad Gräfe
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1301, USA.
 */

#include "authorization-greylisting.h"

#include "prefs.h"

#include <debug.h>

#include <gtk/gtk.h>
#include <gtkblist.h>
#include <gtkutils.h>

extern PurplePlugin *plugin;

static GtkWidget *make_info_widget(const gchar *markup, gchar *stock_id, gboolean indent) {
	GtkWidget *infobox, *label, *img, *align;

	if(!markup) return NULL;

	infobox = gtk_hbox_new(FALSE, 5);

	if(indent) {
		label = gtk_label_new("");
		gtk_box_pack_start(GTK_BOX(infobox), label, FALSE, FALSE, 10);
	}

	if(stock_id) {
		align = gtk_alignment_new(0.5, 0, 0, 0); /* align img to the top of the space */
		gtk_box_pack_start(GTK_BOX(infobox), align, FALSE, FALSE, 0);

		img = gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_MENU);
		gtk_container_add(GTK_CONTAINER(align), img);
	}

	label = gtk_label_new(NULL);
	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	gtk_label_set_markup(GTK_LABEL(label), markup);
	gtk_box_pack_start(GTK_BOX(infobox), label, FALSE, FALSE, 0);

	return infobox;
}

typedef struct _config_frame {
	GtkWidget *frame;
	GtkWidget *textview;
	GtkWidget *spinbutton;
} ConfigFrame;
static ConfigFrame config_frame;

static void config_frame_detroy_cb(GtkWidget *w, gpointer user_data){
	GtkTextBuffer *buffer;
	GtkTextIter begin, end;
	gchar *newtext, *oldtext;

	purple_debug_info(PLUGIN_STATIC_NAME, "config_frame_detroyed_cb()\n");

	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(config_frame.textview));
	gtk_text_buffer_get_start_iter(buffer, &begin);
	gtk_text_buffer_get_end_iter(buffer, &end);
	newtext = gtk_text_buffer_get_text(buffer, &begin, &end, TRUE);

	oldtext = prefs_get_deny_message();
	
	if(purple_utf8_strcasecmp(newtext, oldtext) != 0) {
		if(purple_prefs_exists(PLUGIN_PREFS_PREFIX "/deny_message")) {
			purple_prefs_set_string(PLUGIN_PREFS_PREFIX "/deny_message", newtext);
		} else {
			purple_prefs_add_string(PLUGIN_PREFS_PREFIX "/deny_message", newtext);
		}
	}

	purple_prefs_set_int(PLUGIN_PREFS_PREFIX "/timeout", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(config_frame.spinbutton)));

}


GtkWidget *get_config_frame(PurplePlugin *plugin) {
	GtkWidget *ret, *textview, *vbox, *infobox, *spin, *hbox, *label;
	GtkTextBuffer *buffer;
	GtkAdjustment *adjustment;
	gchar *msg;

	ret = gtk_vbox_new(FALSE, 5);
	gtk_container_set_border_width(GTK_CONTAINER(ret), 12);

	infobox = make_info_widget(_("Please note that the plugin won't work with other protocols then ICQ and AIM."), GTK_STOCK_DIALOG_INFO, FALSE);
	gtk_box_pack_start(GTK_BOX(ret), infobox, FALSE, FALSE, 0);

	vbox = pidgin_make_frame(ret, _("Deny message:"));

	textview = gtk_text_view_new();
	gtk_widget_set_size_request(textview, 350, 150);
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(textview), GTK_WRAP_WORD_CHAR);
	gtk_box_pack_start(GTK_BOX(vbox), textview, FALSE, FALSE, 0);

	msg = prefs_get_deny_message();
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
	gtk_text_buffer_set_text(buffer, msg, -1);
	g_free(msg);

	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(ret), hbox, FALSE, FALSE, 0);

	label = gtk_label_new("");
	gtk_label_set_markup(GTK_LABEL(label), _("<b>Greylisting timeout:</b>"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

	adjustment = (GtkAdjustment *) gtk_adjustment_new(1.0 * purple_prefs_get_int(PLUGIN_PREFS_PREFIX "/timeout"), 2.0, 60.0, 1.0, 1.0, 0.0);
	spin = gtk_spin_button_new(adjustment, 1.0, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, FALSE, 0);

	label = gtk_label_new(_("minutes"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);



	g_signal_connect(G_OBJECT(ret), "destroy", G_CALLBACK(config_frame_detroy_cb), NULL);

	config_frame.frame = ret;
	config_frame.textview = textview;
	config_frame.spinbutton = spin;

	gtk_widget_show_all(ret);

	return ret;
}

gchar *prefs_get_deny_message(void) {
	gchar *msg;

	if(purple_prefs_exists(PLUGIN_PREFS_PREFIX "/deny_message")) {
		msg = g_strdup(purple_prefs_get_string(PLUGIN_PREFS_PREFIX "/deny_message"));
	} else {
		msg = g_strdup_printf(_("Your authorization request has been denied automatically and your account has been greylisted.\n\nPlease re-request the authorization within %d minutes in order to get it displayed the normal way."), purple_prefs_get_int(PLUGIN_PREFS_PREFIX "/timeout"));
	}

	return msg;

}

void init_prefs(void) {
	purple_prefs_add_none(PLUGIN_PREFS_PREFIX);

	purple_prefs_add_int(PLUGIN_PREFS_PREFIX "/timeout", 10);
}

